@@ -1,8 +1,31 @@
+require 'rubygems'
+require 'rubygems/config_file'
+
 module Bundler
   class RubygemsIntegration
-    def initialize
-      # Work around a RubyGems bug
-      configuration
+
+    def self.version
+      @version ||= Gem::Version.new(Gem::VERSION)
+    end
+
+    def self.provides?(req_str)
+      Gem::Requirement.new(req_str).satisfied_by?(version)
+    end
+
+    def version
+      self.class.version
+    end
+
+    def provides?(req_str)
+      self.class.provides?(req_str)
+    end
+
+    def build_args
+      Gem::Command.build_args
+    end
+
+    def build_args=(args)
+      Gem::Command.build_args = args
     end
 
     def loaded_specs(name)
@@ -23,6 +46,10 @@ module Bundler
 
     def configuration
       Gem.configuration
+    rescue Gem::SystemExitException => e
+      Bundler.ui.error "#{e.class}: #{e.message}"
+      Bundler.ui.trace e
+      raise Gem::SystemExitException
     end
 
     def ruby_engine
@@ -38,6 +65,11 @@ module Bundler
     end
 
     def sources=(val)
+      # Gem.configuration creates a new Gem::ConfigFile, which by default will read ~/.gemrc
+      # If that file exists, its settings (including sources) will overwrite the values we
+      # are about to set here. In order to avoid that, we force memoizing the config file now.
+      configuration
+
       Gem.sources = val
     end
 
@@ -61,10 +93,30 @@ module Bundler
       Gem.path
     end
 
+    def gem_cache
+      gem_path.map{|p| File.expand_path("cache", p) }
+    end
+
+    def spec_cache_dirs
+      @spec_cache_dirs ||= begin
+        dirs = gem_path.map {|dir| File.join(dir, 'specifications')}
+        dirs << Gem.spec_cache_dir if Gem.respond_to?(:spec_cache_dir) # Not in Rubygems 2.0.3 or earlier
+        dirs.uniq.select {|dir| File.directory? dir}
+      end
+    end
+
     def marshal_spec_dir
       Gem::MARSHAL_SPEC_DIR
     end
 
+    def config_map
+      Gem::ConfigMap
+    end
+
+    def repository_subdirectories
+      %w[cache doc gems specifications]
+    end
+
     def clear_paths
       Gem.clear_paths
     end
@@ -73,10 +125,6 @@ module Bundler
       Gem.bin_path(gem, bin, ver)
     end
 
-    def refresh
-      Gem.refresh
-    end
-
     def preserve_paths
       # this is a no-op outside of Rubygems 1.8
       yield
@@ -87,25 +135,80 @@ module Bundler
     end
 
     def fetch_specs(all, pre, &blk)
-      Gem::SpecFetcher.new.list(all, pre).each(&blk)
+      specs = Gem::SpecFetcher.new.list(all, pre)
+      specs.each { yield } if block_given?
+      specs
+    end
+
+    def fetch_prerelease_specs
+      fetch_specs(false, true)
+    rescue Gem::RemoteFetcher::FetchError
+      [] # if we can't download them, there aren't any
+    end
+
+    def fetch_all_remote_specs
+      # Fetch all specs, minus prerelease specs
+      spec_list = fetch_specs(true, false)
+      # Then fetch the prerelease specs
+      fetch_prerelease_specs.each {|k, v| spec_list[k] += v }
+
+      return spec_list
     end
 
     def with_build_args(args)
-      old_args = Gem::Command.build_args
+      old_args = self.build_args
       begin
-        Gem::Command.build_args = args
+        self.build_args = args
         yield
       ensure
-        Gem::Command.build_args = old_args
+        self.build_args = old_args
       end
     end
 
-    def spec_from_gem(path)
-      Gem::Format.from_file_by_path(path).spec
+    def gem_from_path(path, policy = nil)
+      require 'rubygems/format'
+      Gem::Format.from_file_by_path(path, policy)
+    end
+
+    def spec_from_gem(path, policy = nil)
+      require 'rubygems/security'
+      gem_from_path(path, security_policies[policy]).spec
+    rescue Gem::Package::FormatError
+      raise GemspecError, "Could not read gem at #{path}. It may be corrupted."
+    rescue Exception, Gem::Exception, Gem::Security::Exception => e
+      if e.is_a?(Gem::Security::Exception) ||
+          e.message =~ /unknown trust policy|unsigned gem/i ||
+          e.message =~ /couldn't verify (meta)?data signature/i
+        raise SecurityError,
+          "The gem #{File.basename(path, '.gem')} can't be installed because " \
+          "the security policy didn't allow it, with the message: #{e.message}"
+      else
+        raise e
+      end
+    end
+
+    def build(spec, skip_validation = false)
+      require 'rubygems/builder'
+      Gem::Builder.new(spec).build
+    end
+
+    def build_gem(gem_dir, spec)
+      SharedHelpers.chdir(gem_dir) { build(spec) }
     end
 
     def download_gem(spec, uri, path)
-      Gem::RemoteFetcher.fetcher.download(spec, uri, path)
+      uri = Bundler::Source.mirror_for(uri)
+      fetcher = Gem::RemoteFetcher.new(configuration[:http_proxy])
+      fetcher.download(spec, uri, path)
+    end
+
+    def security_policies
+      @security_policies ||= begin
+        require 'rubygems/security'
+        Gem::Security::Policies
+      rescue LoadError, NameError
+        {}
+      end
     end
 
     def reverse_rubygems_kernel_mixin
@@ -121,6 +224,8 @@ module Bundler
     end
 
     def replace_gem(specs)
+      reverse_rubygems_kernel_mixin
+
       executables = specs.map { |s| s.executables }.flatten
 
       ::Kernel.send(:define_method, :gem) do |dep, *reqs|
@@ -161,31 +266,9 @@ module Bundler
       end
     end
 
-    if defined? ::Deprecate
-      Deprecate = ::Deprecate
-    elsif defined? Gem::Deprecate
-      Deprecate = Gem::Deprecate
-    else
-      class Deprecate
-        def skip_during; yield; end
-      end
-    end
-
-    def stub_source_index137(specs)
-      # Rubygems versions lower than 1.7 use SourceIndex#from_gems_in
-      source_index_class = (class << Gem::SourceIndex ; self ; end)
-      source_index_class.send(:remove_method, :from_gems_in)
-      source_index_class.send(:define_method, :from_gems_in) do |*args|
-        source_index = Gem::SourceIndex.new
-        source_index.spec_dirs = *args
-        source_index.add_specs(*specs)
-        source_index
-      end
-    end
-
-    def stub_source_index170(specs)
+    def stub_source_index(specs)
       Gem::SourceIndex.send(:alias_method, :old_initialize, :initialize)
-      Gem::SourceIndex.send(:define_method, :initialize) do |*args|
+      redefine_method(Gem::SourceIndex, :initialize) do |*args|
         @gems = {}
         # You're looking at this thinking: Oh! This is how I make those
         # rubygems deprecations go away!
@@ -208,8 +291,7 @@ module Bundler
     # +specs+
     def replace_bin_path(specs)
       gem_class = (class << Gem ; self ; end)
-      gem_class.send(:remove_method, :bin_path)
-      gem_class.send(:define_method, :bin_path) do |name, *args|
+      redefine_method(gem_class, :bin_path) do |name, *args|
         exec_name = args.first
 
         if exec_name == 'bundle'
@@ -220,6 +302,11 @@ module Bundler
 
         if exec_name
           spec = specs.find { |s| s.executables.include?(exec_name) }
+          unless spec.name == name
+            warn "Bundler is using a binstub that was created for a different gem.\n" \
+              "This is deprecated, in future versions you may need to `bundle binstub #{name}` " \
+              "to work around a system/bundle conflict."
+          end
           spec or raise Gem::Exception, "can't find executable #{exec_name}"
         else
           spec = specs.find  { |s| s.name == name }
@@ -236,15 +323,12 @@ module Bundler
     # we don't #refresh, so stub it out.
     def replace_refresh
       gem_class = (class << Gem ; self ; end)
-      gem_class.send(:remove_method, :refresh)
-      gem_class.send(:define_method, :refresh) { }
+      redefine_method(gem_class, :refresh) { }
     end
 
     # Replace or hook into Rubygems to provide a bundlerized view
     # of the world.
     def replace_entrypoints(specs)
-      reverse_rubygems_kernel_mixin
-
       replace_gem(specs)
 
       stub_rubygems(specs)
@@ -258,7 +342,7 @@ module Bundler
     # This backports the correct segment generation code from Rubygems 1.4+
     # by monkeypatching it into the method in Rubygems 1.3.6 and 1.3.7.
     def backport_segment_generation
-      Gem::Version.send(:define_method, :segments) do
+      redefine_method(Gem::Version, :segments) do
         @segments ||= @version.scan(/[0-9]+|[a-z]+/i).map do |s|
           /^\d+$/ =~ s ? s.to_i : s
         end
@@ -267,7 +351,7 @@ module Bundler
 
     # This backport fixes the marshaling of @segments.
     def backport_yaml_initialize
-      Gem::Version.send(:define_method, :yaml_initialize) do |tag, map|
+      redefine_method(Gem::Version, :yaml_initialize) do |tag, map|
         @version = map['version']
         @segments = nil
         @hash = nil
@@ -277,32 +361,39 @@ module Bundler
     # This backports base_dir which replaces installation path
     # Rubygems 1.8+
     def backport_base_dir
-      Gem::Specification.send(:define_method, :base_dir) do
+      redefine_method(Gem::Specification, :base_dir) do
         return Gem.dir unless loaded_from
         File.dirname File.dirname loaded_from
       end
     end
 
     def backport_cache_file
-      Gem::Specification.send(:define_method, :cache_dir) do
+      redefine_method(Gem::Specification, :cache_dir) do
         @cache_dir ||= File.join base_dir, "cache"
       end
 
-      Gem::Specification.send(:define_method, :cache_file) do
+      redefine_method(Gem::Specification, :cache_file) do
         @cache_file ||= File.join cache_dir, "#{full_name}.gem"
       end
     end
 
     def backport_spec_file
-      Gem::Specification.send(:define_method, :spec_dir) do
+      redefine_method(Gem::Specification, :spec_dir) do
         @spec_dir ||= File.join base_dir, "specifications"
       end
 
-      Gem::Specification.send(:define_method, :spec_file) do
+      redefine_method(Gem::Specification, :spec_file) do
         @spec_file ||= File.join spec_dir, "#{full_name}.gemspec"
       end
     end
 
+    def redefine_method(klass, method, &block)
+      if klass.instance_methods(false).include?(method)
+        klass.send(:remove_method, method)
+      end
+      klass.send(:define_method, method, &block)
+    end
+
     # Rubygems 1.4 through 1.6
     class Legacy < RubygemsIntegration
       def initialize
@@ -314,7 +405,14 @@ module Bundler
       end
 
       def stub_rubygems(specs)
-        stub_source_index137(specs)
+        # Rubygems versions lower than 1.7 use SourceIndex#from_gems_in
+        source_index_class = (class << Gem::SourceIndex ; self ; end)
+        source_index_class.send(:define_method, :from_gems_in) do |*args|
+          source_index = Gem::SourceIndex.new
+          source_index.spec_dirs = *args
+          source_index.add_specs(*specs)
+          source_index
+        end
       end
 
       def all_specs
@@ -337,11 +435,11 @@ module Bundler
     # Rubygems 1.7
     class Transitional < Legacy
       def stub_rubygems(specs)
-        stub_source_index170(specs)
+        stub_source_index(specs)
       end
     end
 
-    # Rubygems 1.8.5
+    # Rubygems 1.8.5-1.8.19
     class Modern < RubygemsIntegration
       def stub_rubygems(specs)
         Gem::Specification.all = specs
@@ -350,7 +448,7 @@ module Bundler
           Gem::Specification.all = specs
         }
 
-        stub_source_index170(specs)
+        stub_source_index(specs)
       end
 
       def all_specs
@@ -374,15 +472,97 @@ module Bundler
       end
     end
 
+    # Rubygems 1.8.20+
+    class MoreModern < Modern
+      # Rubygems 1.8.20 and adds the skip_validation parameter, so that's
+      # when we start passing it through.
+      def build(spec, skip_validation = false)
+        require 'rubygems/builder'
+        Gem::Builder.new(spec).build(skip_validation)
+      end
+    end
+
+    # Rubygems 2.0
+    class Future < RubygemsIntegration
+      def stub_rubygems(specs)
+        Gem::Specification.all = specs
+
+        Gem.post_reset do
+          Gem::Specification.all = specs
+        end
+      end
+
+      def all_specs
+        Gem::Specification.to_a
+      end
+
+      def find_name(name)
+        Gem::Specification.find_all_by_name name
+      end
+
+      def fetch_specs(source, name)
+        path = source + "#{name}.#{Gem.marshal_version}.gz"
+        string = Gem::RemoteFetcher.fetcher.fetch_path(path)
+        Bundler.load_marshal(string)
+      rescue Gem::RemoteFetcher::FetchError => e
+        # it's okay for prerelease to fail
+        raise e unless name == "prerelease_specs"
+      end
+
+      def fetch_all_remote_specs
+        # Since SpecFetcher now returns NameTuples, we just fetch directly
+        # and unmarshal the array ourselves.
+        hash = {}
+
+        Gem.sources.each do |source|
+          source = URI.parse(source.to_s) unless source.is_a?(URI)
+          hash[source] = fetch_specs(source, "specs")
+
+          pres = fetch_specs(source, "prerelease_specs")
+          hash[source].push(*pres) if pres && !pres.empty?
+        end
+
+        hash
+      end
+
+      def download_gem(spec, uri, path)
+        require 'resolv'
+        uri = Bundler::Source.mirror_for(uri)
+        proxy, dns = configuration[:http_proxy], Resolv::DNS.new
+        fetcher = Gem::RemoteFetcher.new(proxy, dns)
+        fetcher.download(spec, uri, path)
+      end
+
+      def gem_from_path(path, policy = nil)
+        require 'rubygems/package'
+        p = Gem::Package.new(path)
+        p.security_policy = policy if policy
+        return p
+      end
+
+      def build(spec, skip_validation = false)
+        require 'rubygems/package'
+        Gem::Package.build(spec, skip_validation)
+      end
+
+      def repository_subdirectories
+        Gem::REPOSITORY_SUBDIRECTORIES
+      end
+    end
+
   end
 
-  if Gem::Version.new(Gem::VERSION) >= Gem::Version.new('1.8.5')
+  if RubygemsIntegration.provides?(">= 1.99.99")
+    @rubygems = RubygemsIntegration::Future.new
+  elsif RubygemsIntegration.provides?('>= 1.8.20')
+    @rubygems = RubygemsIntegration::MoreModern.new
+  elsif RubygemsIntegration.provides?('>= 1.8.5')
     @rubygems = RubygemsIntegration::Modern.new
-  elsif Gem::Version.new(Gem::VERSION) >= Gem::Version.new('1.8.0')
+  elsif RubygemsIntegration.provides?('>= 1.8.0')
     @rubygems = RubygemsIntegration::AlmostModern.new
-  elsif Gem::Version.new(Gem::VERSION) >= Gem::Version.new('1.7.0')
+  elsif RubygemsIntegration.provides?('>= 1.7.0')
     @rubygems = RubygemsIntegration::Transitional.new
-  elsif Gem::Version.new(Gem::VERSION) >= Gem::Version.new('1.4.0')
+  elsif RubygemsIntegration.provides?('>= 1.4.0')
     @rubygems = RubygemsIntegration::Legacy.new
   else # Rubygems 1.3.6 and 1.3.7
     @rubygems = RubygemsIntegration::Ancient.new
