@@ -314,7 +314,7 @@ describe "Bundler.setup" do
 
     it "provides a useful exception when the git repo is not checked out yet" do
       run "1", :expect_err => true
-      expect(err).to include("#{lib_path('rack-1.0.0')} (at master) is not checked out. Please run `bundle install`")
+      expect(err).to match(/the git source #{lib_path('rack-1.0.0')} is not yet checked out. Please run `bundle install`/i)
     end
 
     it "does not hit the git binary if the lockfile is available and up to date" do
@@ -550,7 +550,7 @@ describe "Bundler.setup" do
           end
         R
 
-        expect(out).to eq("You have already activated thin 1.1, but your Gemfile requires thin 1.0. Using bundle exec may solve this.")
+        expect(out).to eq("You have already activated thin 1.1, but your Gemfile requires thin 1.0. Prepending `bundle exec` to your command may solve this.")
       end
 
       it "version_requirement is now deprecated in rubygems 1.4.0+" do
@@ -594,7 +594,7 @@ describe "Bundler.setup" do
 
     run <<-R
       Gem.loaded_specs.each do |n, s|
-        puts "FAIL" unless String === s.loaded_from
+        puts "FAIL" unless s.loaded_from.is_a?(String)
       end
     R
 
@@ -631,7 +631,7 @@ describe "Bundler.setup" do
     expect(out).to eq("yay")
   end
 
-  it "ignores Gem.refresh" do
+  it "stubs out Gem.refresh so it does not reveal system gems" do
     system_gems "rack-1.0.0"
 
     install_gemfile <<-G
@@ -640,11 +640,12 @@ describe "Bundler.setup" do
     G
 
     run <<-R
+      puts Bundler.rubygems.find_name("rack").inspect
       Gem.refresh
       puts Bundler.rubygems.find_name("rack").inspect
     R
 
-    expect(out).to eq("[]")
+    expect(out).to eq("[]\n[]")
   end
 
   describe "when a vendored gem specification uses the :path option" do
